<?php

namespace Slovocast\Domain\Aggregate;

use Slovocast\Domain\Entity\Episode;
use Slovocast\Domain\Entity\Channel;

class ChannelAggregate implements ChannelAggregateInterface
{
    protected Channel $channel;

    /**
     * @var array<Episode> $episodes
     */
    protected array $episodes;

    /**
     * @param Channel $channel
     * @param array<Episode> $episodes
     */
    public function __construct(
        Channel $channel,
        array $episodes
    ) { 
        $this->channel = $channel;
        $this->setEpisodes($episodes);
    }

    public function getChannel(): Channel
    {
        return $this->channel;
    }

    public function hasEpisodes(): bool
    {
        return (bool) count($this->episodes);
    }

    /**
     * @return array<Episode>
     */
    public function getEpisodes(): array
    {
        return $this->episodes;
    }

    public function addEpisode(Episode $episode): void
    {
        $this->episodes[] = $episode;
    }

    /**
     * @param array<Episode> $episodes
     * @return void
     */
    public function setEpisodes(array $episodes): void
    {
        foreach ($episodes as $episode) {
            $this->addEpisode($episode);
        }
    }
}
