<?php

namespace Slovocast;

use League\Config\Configuration;
use Odan\Session\Middleware\SessionStartMiddleware;
use Odan\Session\SessionInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Slim\App;
use Slim\Views\Twig;
use Slim\Views\TwigMiddleware;
use Twig\Error\LoaderError;

class Middlewares
{
    /**
     * Set up all the application middleware. This is where the application
     * should determine the environment the code is running in as well.
     *
     * @param App $app The Slim application
     * @return void
     * @throws NotFoundExceptionInterface
     * @throws ContainerExceptionInterface
     * @throws LoaderError
     */
    public static function setup(App $app): void
    {
        /** @var ContainerInterface $container */
        $container = $app->getContainer();

        /** @var Configuration $config */
        $config = $container->get('config');

        /**
         * Global session start
         */
        $app->add(SessionStartMiddleware::class);

        // Twig
        $templateCache = false;
        $twig = Twig::create(APP_TEMPLATES_DIR, [
            'cache' => $templateCache,
            'debug' => true,
        ]);

        // Add the global variables
        $twig->getEnvironment()
             ->addGlobal('site_name', $config->get('site.name'));
        $twig->getEnvironment()
             ->addGlobal('site_description', $config->get('site.description'));

        /** @var SessionInterface $session */
        $session = $container->get(SessionInterface::class);
        $twig->getEnvironment()->addGlobal('session', $session);

        $app->add(TwigMiddleware::create($app, $twig));

        // Add the error handling middleware
        $app->addErrorMiddleware(true, true, true);
    }
}
