<?php

namespace Slovocast;

use Slim\App;
use Slovocast\Handler\Channel\CreateChannelAction;
use Slovocast\Handler\Channel\Page\CreateChannelPage;
use Slovocast\Handler\HealthCheck;
use Slovocast\Handler\Page\DashboardPage;
use Slovocast\Handler\Page\HomePage;
use Slovocast\Handler\User\LoginUserAction;
use Slovocast\Handler\User\LogoutUserAction;
use Slovocast\Handler\User\Page\LoginUserPage;
use Slovocast\Handler\User\Page\RegisterUserPage;
use Slovocast\Handler\User\RegisterUserAction;
use Slovocast\Middleware\AuthenticatedMiddleware;
use Slovocast\Middleware\FormKeyMiddleware;
use Slovocast\Middleware\VerifyPasswordMiddleware;

class Routes
{
    /**
     * @param App $app Instantiated Application
     * @return void
     */
    public static function setup(App $app): void
    {
        $app->get('/[home]', HomePage::class)->setName("home");
        $app->get('/healthcheck', HealthCheck::class);

        // User Routes
        self::users($app);
        self::dashboard($app);

        // Channel Routes
        self::channels($app);
    }

    /**
     * @param App $app Instantiated Application
     */
    protected static function users(App $app): void
    {
        $app->get('/register', RegisterUserPage::class)
            ->add(FormKeyMiddleware::class)
            ->setName('user-register-page');
        $app->post('/register', RegisterUserAction::class)
            ->add(FormKeyMiddleware::class)
            ->add(VerifyPasswordMiddleware::class)
            ->setName('user-register-action');

        $app->get('/login', LoginUserPage::class)
            ->setName('user-login-page');
        $app->post('/login', LoginUserAction::class)
            ->setName('user-login-action');

        $app->get('/logout', LogoutUserAction::class)
            ->setName('user-logout');
    }

    /**
     * @param App $app Instantiated Application
     */
    protected static function channels(App $app): void
    {
        $app->get('/channel/create', CreateChannelPage::class)
            ->add(FormKeyMiddleware::class)
            ->add(AuthenticatedMiddleware::class)
            ->setName('channel-create-page');
        $app->post('/channel/create', CreateChannelAction::class)
            ->add(FormKeyMiddleware::class)
            ->add(AuthenticatedMiddleware::class)
            ->setName("channel-create-action");
    }

    /**
     * @param App $app Instance of the application
     */
    protected static function dashboard(App $app): void
    {
        $app->get('/dashboard', DashboardPage::class)
            ->add(AuthenticatedMiddleware::class)
            ->setName('dashboard');
    }
}
